function b = sgreflection(miller,sg)
% ***********************************************
% Copyright (c) 2017 UChicago Argonne, LLC
% See LICENSE file.
% ***********************************************
%
% SGREFLECTION Determine the reflection condition of  a miller index set.
%   B=SGREFLECTION(MILLER,SG) determine if the diffraction is allowed for a
%   miller index set [h,k,l] for space group number SG. It returns 1 if
%   reflection is allowed and 0 if forbidden.
% 
%   Reference: International Tables for Crystallography Volume A. Only 
%   general rules are applied. In present of multiple cell choices, the
%   first one is used.    

%   Zhang Jiang @8ID/APS/ANL
%   $Revision: 1.0 $  $Date: 2012/08/08 $

if floor(sg) ~= sg || sg<1 || sg>230
    error('Invalid space group');
end

% --- miller index
miller =  miller(:)';         % 1x3 array
h = miller(1);
k = miller(2);
l = miller(3);
ihk = -(h+k);               % for hexagonal crystal system
m = logical(miller);
mi = logical([h,k,ihk,l]);

% --- direct beam
if isequal(m,[0 0 0]);
    b = 1;
    return;
end

% --- reflection beam
b = 0;
switch sg
    case 1
        b = 1;
    case 2
        b = 1;
    case 3
        b = 1;
    case 4
        if  ...
                (isequal(m,[0 1 0]) && iseven(k))
            b = 1;
        end
    case 5
        if  ...
                (isequal(m,[1 1 1]) && iseven(h+k)) || ...
                (isequal(m,[1 0 1]) && iseven(h)) || ...
                (isequal(m,[0 1 1]) && iseven(k)) || ...
                (isequal(m,[1 1 0]) && iseven(h+k)) || ...
                (isequal(m,[0 1 0]) && iseven(k)) || ...
                (isequal(m,[1 0 0]) && iseven(h))
            b = 1;
        end
    case 6
        b = 1;
    case 7
        if  ...
                (isequal(m,[1 0 1]) && iseven(l)) || ...
                (isequal(m,[0 0 1]) && iseven(l))
            b = 1;
        end
    case 8
        if ...
                (isequal(m,[1 1 1]) && iseven(h+k)) || ...
                (isequal(m,[1 0 1]) && iseven(h)) || ...
                (isequal(m,[0 1 1]) && iseven(k)) || ...
                (isequal(m,[1 1 0]) && iseven(h+k)) || ...
                (isequal(m,[0 1 0]) && iseven(k)) || ...
                (isequal(m,[1 0 0]) && iseven(h))
            b = 1;
        end
    case 9
        if  ...
                (isequal(m,[1 1 1]) && iseven(h+k)) || ...
                (isequal(m,[1 0 1]) && iseven(h)) && iseven(l) || ...
                (isequal(m,[0 1 1]) && iseven(k)) || ...
                (isequal(m,[1 1 0]) && iseven(h+k)) || ...
                (isequal(m,[0 1 0]) && iseven(k)) || ...
                (isequal(m,[1 0 0]) && iseven(h)) || ...
                (isequal(m,[0 0 1]) && iseven(l))
            b = 1;
        end
    case 10
        b = 1;
    case 11
        if  ...
                (isequal(m,[0 1 0]) && iseven(k))
            b = 1;
        end
    case 12
        if  ...
                (isequal(m,[1 1 1]) && iseven(h+k)) || ...
                (isequal(m,[1 0 1]) && iseven(h)) || ...
                (isequal(m,[0 1 1]) && iseven(k)) || ...
                (isequal(m,[1 1 0]) && iseven(h+k)) || ...
                (isequal(m,[0 1 0]) && iseven(k)) || ...
                (isequal(m,[1 0 0]) && iseven(h))
            b = 1;
        end
    case 13
        if  ...
                (isequal(m,[1 0 1]) && iseven(l)) || ...
                (isequal(m,[0 0 1]) && iseven(l))
            b = 1;
        end
    case 14
        if ...
                (isequal(m,[1 0 1]) && iseven(l)) || ...
                (isequal(m,[0 1 0]) && iseven(k)) || ...
                (isequal(m,[0 0 1]) && iseven(l))
            b = 1;
        end
    case 15
        if ...
                (isequal(m,[1 1 1]) && iseven(h+k)) || ...
                (isequal(m,[1 0 1]) && iseven(h)) && iseven(l) || ...
                (isequal(m,[0 1 1]) && iseven(k)) || ...
                (isequal(m,[1 1 0]) && iseven(h+k)) || ...
                (isequal(m,[0 1 0]) && iseven(k)) || ...
                (isequal(m,[1 0 0]) && iseven(h)) || ...
                (isequal(m,[0 0 1]) && iseven(l))
            b = 1;
        end
    case 16
        b = 1;
    case 17
        if ...
                (isequal(m,[0 0 1]) && iseven(l))
            b = 1;
        end
    case 18
        if ...
                (isequal(m,[1 0 0]) && iseven(h)) || ...
                (isequal(m,[0 1 0]) && iseven(k))
            b = 1;
        end
    case 19
        if ...
                (isequal(m,[1 0 0]) && iseven(h)) || ...
                (isequal(m,[0 1 0]) && iseven(k)) || ...
                (isequal(m,[0 0 1]) && iseven(l))
            b = 1;
        end
    case 20
        if  ...
                (isequal(m,[1 1 1]) && iseven(h+k)) || ...
                (isequal(m,[0 1 1]) && iseven(k)) || ...
                (isequal(m,[1 0 1]) && iseven(h)) || ...
                (isequal(m,[1 1 0]) && iseven(h+k)) || ...
                (isequal(m,[1 0 0]) && iseven(h)) || ...
                (isequal(m,[0 1 0]) && iseven(k)) || ...
                (isequal(m,[0 0 1]) && iseven(l))
            b = 1;
        end
    case 21
        if ...
                (isequal(m,[1 1 1]) && iseven(h+k)) || ...
                (isequal(m,[0 1 1]) && iseven(k)) || ...
                (isequal(m,[1 0 1]) && iseven(h)) || ...
                (isequal(m,[1 1 0]) && iseven(h+k)) || ...
                (isequal(m,[1 0 0]) && iseven(h)) || ...
                (isequal(m,[0 1 0]) && iseven(k))
            b = 1;
        end
    case 22
        if ...
                (isequal(m,[1 1 1]) && iseven(h+k) && iseven(h+l) && iseven(k+l)) || ...
                (isequal(m,[0 1 1]) && iseven(k) && iseven(l)) || ...
                (isequal(m,[1 0 1]) && iseven(h) && iseven(l)) || ...
                (isequal(m,[1 1 0]) && iseven(h) && iseven(k)) || ...
                (isequal(m,[1 0 0]) && iseven(h)) || ...
                (isequal(m,[0 1 0]) && iseven(k)) || ...
                (isequal(m,[0 0 1]) && iseven(l))
            b = 1;
        end
    case 23
        if ...
                (isequal(m,[1 1 1]) && iseven(h+k+l)) || ...
                (isequal(m,[0 1 1]) && iseven(k+l)) || ...
                (isequal(m,[1 0 1]) && iseven(h+l)) || ...
                (isequal(m,[1 1 0]) && iseven(h+k)) || ...
                (isequal(m,[1 0 0]) && iseven(h)) || ...
                (isequal(m,[0 1 0]) && iseven(k)) || ...
                (isequal(m,[0 0 1]) && iseven(l))
            b = 1;
        end
    case 24
        if ...
                (isequal(m,[1 1 1]) && iseven(h+k+l)) || ...
                (isequal(m,[0 1 1]) && iseven(k+l)) || ...
                (isequal(m,[1 0 1]) && iseven(h+l)) || ...
                (isequal(m,[1 1 0]) && iseven(h+k)) || ...
                (isequal(m,[1 0 0]) && iseven(h)) || ...
                (isequal(m,[0 1 0]) && iseven(k)) || ...
                (isequal(m,[0 0 1]) && iseven(l))
            b = 1;
        end
    case 25
        b = 1;
    case 26
        if ...
                (isequal(m,[1 0 1]) && iseven(l)) || ...
                (isequal(m,[0 0 1]) && iseven(l))
            b = 1;
        end
    case 27
        if ...
                (isequal(m,[0 1 1]) && iseven(l)) || ...
                (isequal(m,[1 0 1]) && iseven(l)) || ...
                (isequal(m,[0 0 1]) && iseven(l))
            b = 1;
        end
    case 28
        if ...
                (isequal(m,[1 0 1]) && iseven(h)) || ...
                (isequal(m,[1 0 0]) && iseven(h))
            b = 1;
        end
    case 29
        if ...
                (isequal(m,[0 1 1]) && iseven(l)) || ...
                (isequal(m,[1 0 1]) && iseven(h)) || ...
                (isequal(m,[1 0 0]) && iseven(h)) || ...
                (isequal(m,[0 0 1]) && iseven(l))
            b = 1;
        end
    case 30
        if ...
                (isequal(m,[0 1 1]) && iseven(k+l)) || ...
                (isequal(m,[1 0 1]) && iseven(l)) || ...
                (isequal(m,[0 1 0]) && iseven(k)) || ...
                (isequal(m,[0 0 1]) && iseven(l))
            b = 1;
        end
    case 31
        if ...
                (isequal(m,[1 0 1]) && iseven(h+l)) || ...
                (isequal(m,[1 0 0]) && iseven(h)) || ...
                (isequal(m,[0 0 1]) && iseven(l))
            b = 1;
        end
    case 32
        if ...
                (isequal(m,[0 1 1]) && iseven(k)) || ...
                (isequal(m,[1 0 1]) && iseven(h)) || ...
                (isequal(m,[1 0 0]) && iseven(h)) || ...
                (isequal(m,[0 1 0]) && iseven(k))
            b = 1;
        end
    case 33
        if ...
                (isequal(m,[0 1 1]) && iseven(k+l)) || ...
                (isequal(m,[1 0 1]) && iseven(h)) || ...
                (isequal(m,[1 0 0]) && iseven(h)) || ...
                (isequal(m,[0 1 0]) && iseven(k)) || ...
                (isequal(m,[0 0 1]) && iseven(l))
            b = 1;
        end
    case 34
        if ...
                (isequal(m,[0 1 1]) && iseven(k+l)) || ...
                (isequal(m,[1 0 1]) && iseven(h+l)) || ...
                (isequal(m,[1 0 0]) && iseven(h)) || ...
                (isequal(m,[0 1 0]) && iseven(k)) || ...
                (isequal(m,[0 0 1]) && iseven(l))
            b = 1;
        end
    case 35
        if ...
                (isequal(m,[1 1 1]) && iseven(h+k)) || ...
                (isequal(m,[0 1 1]) && iseven(k)) || ...
                (isequal(m,[1 0 1]) && iseven(h)) || ...
                (isequal(m,[1 1 0]) && iseven(h+k)) || ...
                (isequal(m,[1 0 0]) && iseven(h)) || ...
                (isequal(m,[0 1 0]) && iseven(k))
            b = 1;
        end
    case 36
        if ...
                (isequal(m,[1 1 1]) && iseven(h+k)) || ...
                (isequal(m,[0 1 1]) && iseven(k)) || ...
                (isequal(m,[1 0 1]) && iseven(h) && iseven(l)) || ...
                (isequal(m,[1 1 0]) && iseven(h+k)) || ...
                (isequal(m,[1 0 0]) && iseven(h)) || ...
                (isequal(m,[0 1 0]) && iseven(k)) || ...
                (isequal(m,[0 0 1]) && iseven(l))
            b = 1;
        end
    case 37
        if ...
                (isequal(m,[1 1 1]) && iseven(h+k)) || ...
                (isequal(m,[0 1 1]) && iseven(k) && iseven(l)) || ...
                (isequal(m,[1 0 1]) && iseven(h) && iseven(l)) || ...
                (isequal(m,[1 1 0]) && iseven(h+k)) || ...
                (isequal(m,[1 0 0]) && iseven(h)) || ...
                (isequal(m,[0 1 0]) && iseven(k)) || ...
                (isequal(m,[0 0 1]) && iseven(l))
            b = 1;
        end
    case 38
        if ...
                (isequal(m,[1 1 1]) && iseven(k+l)) || ...
                (isequal(m,[0 1 1]) && iseven(k+l)) || ...
                (isequal(m,[1 0 1]) && iseven(l)) || ...
                (isequal(m,[1 1 0]) && iseven(k)) || ...
                (isequal(m,[0 1 0]) && iseven(k)) || ...
                (isequal(m,[0 0 1]) && iseven(l))
            b = 1;
        end
    case 39
        if ...
                (isequal(m,[1 1 1]) && iseven(k+l)) || ...
                (isequal(m,[0 1 1]) && iseven(k) && iseven(l)) || ...
                (isequal(m,[1 0 1]) && iseven(l)) || ...
                (isequal(m,[1 1 0]) && iseven(k)) || ...
                (isequal(m,[0 1 0]) && iseven(k)) || ...
                (isequal(m,[0 0 1]) && iseven(l))
            b = 1;
        end        
    case 40
        if ...
                (isequal(m,[1 1 1]) && iseven(k+l)) || ...
                (isequal(m,[0 1 1]) && iseven(k+l)) || ...
                (isequal(m,[1 0 1]) && iseven(h) && iseven(l)) || ...
                (isequal(m,[1 1 0]) && iseven(k)) || ...
                (isequal(m,[1 0 0]) && iseven(h)) || ...
                (isequal(m,[0 1 0]) && iseven(k)) || ...
                (isequal(m,[0 0 1]) && iseven(l))
            b = 1;
        end        
    case 41
        if ...
                (isequal(m,[1 1 1]) && iseven(k+l)) || ...
                (isequal(m,[0 1 1]) && iseven(k) && iseven(l)) || ...
                (isequal(m,[1 0 1]) && iseven(h) && iseven(l)) || ...
                (isequal(m,[1 1 0]) && iseven(k)) || ...
                (isequal(m,[1 0 0]) && iseven(h)) || ...
                (isequal(m,[0 1 0]) && iseven(k)) || ...
                (isequal(m,[0 0 1]) && iseven(l))
            b = 1;
        end        
    case 42
        if ...
                (isequal(m,[1 1 1]) && iseven(h+k) && iseven(h+l) && iseven(k+l)) || ...
                (isequal(m,[0 1 1]) && iseven(k) && iseven(l)) || ...
                (isequal(m,[1 0 1]) && iseven(h) && iseven(l)) || ...
                (isequal(m,[1 1 0]) && iseven(h) && iseven(k)) || ...
                (isequal(m,[1 0 0]) && iseven(h)) || ...
                (isequal(m,[0 1 0]) && iseven(k)) || ...
                (isequal(m,[0 0 1]) && iseven(l))
            b = 1;
        end          
    case 43
        if ...
                (isequal(m,[1 1 1]) && iseven(h+k) && iseven(h+l) && iseven(k+l)) || ...
                (isequal(m,[0 1 1]) && iseven((k+l)/2) && iseven(k) && iseven(l)) || ...
                (isequal(m,[1 0 1]) && iseven((h+l)/2) && iseven(h) && iseven(l)) || ...
                (isequal(m,[1 1 0]) && iseven(h) && iseven(k)) || ...
                (isequal(m,[1 0 0]) && iseven(h/2)) || ...
                (isequal(m,[0 1 0]) && iseven(k/2)) || ...
                (isequal(m,[0 0 1]) && iseven(l/2))
            b = 1;
        end               
    case 44
        if ...
                (isequal(m,[1 1 1]) && iseven(h+k+l)) || ...
                (isequal(m,[0 1 1]) && iseven(k+l)) || ...
                (isequal(m,[1 0 1]) && iseven(h+l)) || ...
                (isequal(m,[1 1 0]) && iseven(h+k)) || ...
                (isequal(m,[1 0 0]) && iseven(h)) || ...
                (isequal(m,[0 1 0]) && iseven(k)) || ...
                (isequal(m,[0 0 1]) && iseven(l))
            b = 1;
        end              
    case 45
        if ...
                (isequal(m,[1 1 1]) && iseven(h+k+l)) || ...
                (isequal(m,[0 1 1]) && iseven(k) && iseven(l)) || ...
                (isequal(m,[1 0 1]) && iseven(h) && iseven(l)) || ...
                (isequal(m,[1 1 0]) && iseven(h+k)) || ...
                (isequal(m,[1 0 0]) && iseven(h)) || ...
                (isequal(m,[0 1 0]) && iseven(k)) || ...
                (isequal(m,[0 0 1]) && iseven(l))
            b = 1;
        end             
    case 46
        if ...
                (isequal(m,[1 1 1]) && iseven(h+k+l)) || ...
                (isequal(m,[0 1 1]) && iseven(k+l)) || ...
                (isequal(m,[1 0 1]) && iseven(h) && iseven(l)) || ...
                (isequal(m,[1 1 0]) && iseven(h+k)) || ...
                (isequal(m,[1 0 0]) && iseven(h)) || ...
                (isequal(m,[0 1 0]) && iseven(k)) || ...
                (isequal(m,[0 0 1]) && iseven(l))
            b = 1;
        end             
    case 47
        b = 1;
    case 48
        if ...
                (isequal(m,[0 1 1]) && iseven(k+l)) || ...
                (isequal(m,[1 0 1]) && iseven(h+l)) || ...
                (isequal(m,[1 1 0]) && iseven(h+k)) || ...
                (isequal(m,[1 0 0]) && iseven(h)) || ...
                (isequal(m,[0 1 0]) && iseven(k)) || ...
                (isequal(m,[0 0 1]) && iseven(l))
            b = 1;
        end             
    case 49
        if ...
                (isequal(m,[0 1 1]) && iseven(l)) || ...
                (isequal(m,[1 0 1]) && iseven(l)) || ...
                (isequal(m,[0 0 1]) && iseven(l))
            b = 1;
        end             
    case 50
        if ...
                (isequal(m,[0 1 1]) && iseven(k)) || ...
                (isequal(m,[1 0 1]) && iseven(h)) || ...
                (isequal(m,[1 1 0]) && iseven(h+k)) || ...
                (isequal(m,[1 0 0]) && iseven(h)) || ...
                (isequal(m,[0 1 0]) && iseven(k))
            b = 1;
        end             
    case 51
        if ...
                (isequal(m,[1 1 0]) && iseven(h)) || ...
                (isequal(m,[1 0 0]) && iseven(h))
            b = 1;
        end             
    case 52
        if ...
                (isequal(m,[0 1 1]) && iseven(k+l)) || ...
                (isequal(m,[1 0 1]) && iseven(h+l)) || ...
                (isequal(m,[1 1 0]) && iseven(h)) || ...
                (isequal(m,[1 0 0]) && iseven(h)) || ...
                (isequal(m,[0 1 0]) && iseven(k)) || ...
                (isequal(m,[0 0 1]) && iseven(l))
            b = 1;
        end             
    case 53
        if ...
                (isequal(m,[1 0 1]) && iseven(h+l)) || ...
                (isequal(m,[1 1 0]) && iseven(h)) || ...
                (isequal(m,[1 0 0]) && iseven(h)) || ...
                (isequal(m,[0 0 1]) && iseven(l))
            b = 1;
        end             
    case 54
        if ...
                (isequal(m,[0 1 1]) && iseven(l)) || ...
                (isequal(m,[1 0 1]) && iseven(l)) || ...
                (isequal(m,[1 1 0]) && iseven(h)) || ...
                (isequal(m,[1 0 0]) && iseven(h)) || ...
                (isequal(m,[0 0 1]) && iseven(l))
            b = 1;
        end             
    case 55
        if ...
                (isequal(m,[0 1 1]) && iseven(k)) || ...
                (isequal(m,[1 0 1]) && iseven(h)) || ...
                (isequal(m,[1 0 0]) && iseven(h)) || ...
                (isequal(m,[0 1 0]) && iseven(k))
            b = 1;
        end             
    case 56
        if ...
                (isequal(m,[0 1 1]) && iseven(l)) || ...
                (isequal(m,[1 0 1]) && iseven(l)) || ...
                (isequal(m,[1 1 0]) && iseven(h+k)) || ...
                (isequal(m,[1 0 0]) && iseven(h)) || ...
                (isequal(m,[0 1 0]) && iseven(k)) || ...
                (isequal(m,[0 0 1]) && iseven(l))
            b = 1;
        end             
    case 57
        if ...
                (isequal(m,[0 1 1]) && iseven(k)) || ...
                (isequal(m,[1 0 1]) && iseven(l)) || ...
                (isequal(m,[0 1 0]) && iseven(k)) || ...
                (isequal(m,[0 0 1]) && iseven(l))
            b = 1;
        end             
    case 58
        if ...
                (isequal(m,[0 1 1]) && iseven(k+l)) || ...
                (isequal(m,[1 0 1]) && iseven(h+l)) || ...
                (isequal(m,[1 0 0]) && iseven(h)) || ...
                (isequal(m,[0 1 0]) && iseven(k)) || ...
                (isequal(m,[0 0 1]) && iseven(l))
            b = 1;
        end             
    case 59
        if ...
                (isequal(m,[1 1 0]) && iseven(h+k)) || ...
                (isequal(m,[1 0 0]) && iseven(h)) || ...
                (isequal(m,[0 1 0]) && iseven(k))
            b = 1;
        end             
    case 60
        if ...
                (isequal(m,[0 1 1]) && iseven(k)) || ...
                (isequal(m,[1 0 1]) && iseven(l)) || ...
                (isequal(m,[1 1 0]) && iseven(h+k)) || ...
                (isequal(m,[1 0 0]) && iseven(h)) || ...
                (isequal(m,[0 1 0]) && iseven(k)) || ...
                (isequal(m,[0 0 1]) && iseven(l))
            b = 1;
        end             
    case 61
        if ...
                (isequal(m,[0 1 1]) && iseven(k)) || ...
                (isequal(m,[1 0 1]) && iseven(l)) || ...
                (isequal(m,[1 1 0]) && iseven(h)) || ...
                (isequal(m,[1 0 0]) && iseven(h)) || ...
                (isequal(m,[0 1 0]) && iseven(k)) || ...
                (isequal(m,[0 0 1]) && iseven(l))
            b = 1;
        end             
    case 62
        if ...
                (isequal(m,[0 1 1]) && iseven(k+l)) || ...
                (isequal(m,[1 1 0]) && iseven(h)) || ...
                (isequal(m,[1 0 0]) && iseven(h)) || ...
                (isequal(m,[0 1 0]) && iseven(k)) || ...
                (isequal(m,[0 0 1]) && iseven(l))
            b = 1;
        end             
    case 63
        if ...
                (isequal(m,[1 1 1]) && iseven(h+k)) || ...
                (isequal(m,[0 1 1]) && iseven(k)) || ...
                (isequal(m,[1 0 1]) && iseven(h) && iseven(l)) || ...
                (isequal(m,[1 1 0]) && iseven(h+k)) || ...
                (isequal(m,[1 0 0]) && iseven(h)) || ...
                (isequal(m,[0 1 0]) && iseven(k)) || ...
                (isequal(m,[0 0 1]) && iseven(l))
            b = 1;
        end             
    case 64
        if ...
                (isequal(m,[1 1 1]) && iseven(h+k)) || ...
                (isequal(m,[0 1 1]) && iseven(k)) || ...
                (isequal(m,[1 0 1]) && iseven(h) && iseven(l)) || ...
                (isequal(m,[1 1 0]) && iseven(h) && iseven(k)) || ...
                (isequal(m,[1 0 0]) && iseven(h)) || ...
                (isequal(m,[0 1 0]) && iseven(k)) || ...
                (isequal(m,[0 0 1]) && iseven(l))
            b = 1;
        end             
    case 65
        if ...
                (isequal(m,[1 1 1]) && iseven(h+k)) || ...
                (isequal(m,[0 1 1]) && iseven(k)) || ...
                (isequal(m,[1 0 1]) && iseven(h)) || ...
                (isequal(m,[1 1 0]) && iseven(h+k)) || ...
                (isequal(m,[1 0 0]) && iseven(h)) || ...
                (isequal(m,[0 1 0]) && iseven(k))
            b = 1;
        end             
    case 66
        if ...
                (isequal(m,[1 1 1]) && iseven(h+k)) || ...
                (isequal(m,[0 1 1]) && iseven(k) && iseven(l)) || ...
                (isequal(m,[1 0 1]) && iseven(h) && iseven(l)) || ...
                (isequal(m,[1 1 0]) && iseven(h+k)) || ...
                (isequal(m,[1 0 0]) && iseven(h)) || ...
                (isequal(m,[0 1 0]) && iseven(k)) || ...
                (isequal(m,[0 0 1]) && iseven(l))
            b = 1;
        end             
    case 67
        if ...
                (isequal(m,[1 1 1]) && iseven(h+k)) || ...
                (isequal(m,[0 1 1]) && iseven(k)) || ...
                (isequal(m,[1 0 1]) && iseven(h)) || ...
                (isequal(m,[1 1 0]) && iseven(h) && iseven(k)) || ...
                (isequal(m,[1 0 0]) && iseven(h)) || ...
                (isequal(m,[0 1 0]) && iseven(k))
            b = 1;
        end             
    case 68
        if ...
                (isequal(m,[1 1 1]) && iseven(h+k)) || ...
                (isequal(m,[0 1 1]) && iseven(k) && iseven(l)) || ...
                (isequal(m,[1 0 1]) && iseven(h) && iseven(l)) || ...
                (isequal(m,[1 1 0]) && iseven(h) && iseven(k)) || ...
                (isequal(m,[1 0 0]) && iseven(h)) || ...
                (isequal(m,[0 1 0]) && iseven(k)) || ...
                (isequal(m,[0 0 1]) && iseven(l))
            b = 1;
        end             
    case 69
        if ...
                (isequal(m,[1 1 1]) && iseven(h+k) && iseven(h+l) && iseven(k+l)) || ...
                (isequal(m,[0 1 1]) && iseven(k) && iseven(l)) || ...
                (isequal(m,[1 0 1]) && iseven(h) && iseven(l)) || ...
                (isequal(m,[1 1 0]) && iseven(h) && iseven(k)) || ...
                (isequal(m,[1 0 0]) && iseven(h)) || ...
                (isequal(m,[0 1 0]) && iseven(k)) || ...
                (isequal(m,[0 0 1]) && iseven(l))
            b = 1;
        end             
    case 70
        if ...
                (isequal(m,[1 1 1]) && iseven(h+k) && iseven(h+l) && iseven(k+l)) || ...
                (isequal(m,[0 1 1]) && iseven((k+l)/2) && iseven(k) && iseven(l)) || ...
                (isequal(m,[1 0 1]) && iseven((h+l)/2) && iseven(h) && iseven(l)) || ...
                (isequal(m,[1 1 0]) && iseven((h+k)/2) && iseven(h) && iseven(k)) || ...
                (isequal(m,[1 0 0]) && iseven(h/2)) || ...
                (isequal(m,[0 1 0]) && iseven(k/2)) || ...
                (isequal(m,[0 0 1]) && iseven(l/2))
            b = 1;
        end             
    case 71
        if ...
                (isequal(m,[1 1 1]) && iseven(h+k+l)) || ...
                (isequal(m,[0 1 1]) && iseven(k+l)) || ...
                (isequal(m,[1 0 1]) && iseven(h+l)) || ...
                (isequal(m,[1 1 0]) && iseven(h+k)) || ...
                (isequal(m,[1 0 0]) && iseven(h)) || ...
                (isequal(m,[0 1 0]) && iseven(k)) || ...
                (isequal(m,[0 0 1]) && iseven(l))        
            b = 1;
        end
    case 72
        if ...
                (isequal(m,[1 1 1]) && iseven(h+k+l)) || ...
                (isequal(m,[0 1 1]) && iseven(k) && iseven(l)) || ...
                (isequal(m,[1 0 1]) && iseven(h) && iseven(l)) || ...
                (isequal(m,[1 1 0]) && iseven(h+k)) || ...
                (isequal(m,[1 0 0]) && iseven(h)) || ...
                (isequal(m,[0 1 0]) && iseven(k)) || ...
                (isequal(m,[0 0 1]) && iseven(l))
            b = 1;
        end        
    case 73
        if ...
                (isequal(m,[1 1 1]) && iseven(h+k+l)) || ...
                (isequal(m,[0 1 1]) && iseven(k) && iseven(l)) || ...
                (isequal(m,[1 0 1]) && iseven(h) && iseven(l)) || ...
                (isequal(m,[1 1 0]) && iseven(h) && iseven(k)) || ...
                (isequal(m,[1 0 0]) && iseven(h)) || ...
                (isequal(m,[0 1 0]) && iseven(k)) || ...
                (isequal(m,[0 0 1]) && iseven(l))
            b = 1;
        end         
    case 74
        if ...
                (isequal(m,[1 1 1]) && iseven(h+k+l)) || ...
                (isequal(m,[0 1 1]) && iseven(k+l)) || ...
                (isequal(m,[1 0 1]) && iseven(h+l)) || ...
                (isequal(m,[1 1 0]) && iseven(h) && iseven(k)) || ...
                (isequal(m,[1 0 0]) && iseven(h)) || ...
                (isequal(m,[0 1 0]) && iseven(k)) || ...
                (isequal(m,[0 0 1]) && iseven(l))
            b = 1;
        end         
    case 75
        b = 1;
    case 76
        if ...
                (isequal(m,[0 0 1]) && iseven(l/2))
            b = 1;
        end         
    case 77
        if ...
                (isequal(m,[0 0 1]) && iseven(l))
            b = 1;
        end         
    case 78
        if ...
                (isequal(m,[0 0 1]) && iseven(l/2))
            b = 1;
        end         
    case 79
        if ...
                (isequal(m,[1 1 1]) && iseven(h+k+l)) || ...
                (isequal(m,[1 1 0]) && iseven(h+k)) || ...                
                (isequal(m,[0 1 1]) && iseven(k+l)) || ...
                (isequal(m,[1 1 1]) && iseven(l) && h==k) || ...
                (isequal(m,[0 0 1]) && iseven(l)) || ...                
                (isequal(m,[1 0 0]) && iseven(h))
            b = 1;
        end         
    case 80
        if ...
                (isequal(m,[1 1 1]) && iseven(h+k+l)) || ...
                (isequal(m,[1 1 0]) && iseven(h+k)) || ...                
                (isequal(m,[0 1 1]) && iseven(k+l)) || ...
                (isequal(m,[1 1 1]) && iseven(l) && h==k) || ...
                (isequal(m,[0 0 1]) && iseven(l/2)) || ...                
                (isequal(m,[1 0 0]) && iseven(h))
            b = 1;
        end          
    case 81
        b = 1;
    case 82
        if ...
                (isequal(m,[1 1 1]) && iseven(h+k+l)) || ...
                (isequal(m,[1 1 0]) && iseven(h+k)) || ...                
                (isequal(m,[0 1 1]) && iseven(k+l)) || ...
                (isequal(m,[1 1 1]) && iseven(l) && h==k) || ...
                (isequal(m,[0 0 1]) && iseven(l)) || ...                
                (isequal(m,[1 0 0]) && iseven(h))
            b = 1;
        end          
    case 83
        b = 1;
    case 84
        if ...
                (isequal(m,[0 0 1]) && iseven(l))
            b = 1;
        end          
    case 85
        if ...
                (isequal(m,[1 1 0]) && iseven(h+k)) || ...                
                (isequal(m,[1 0 0]) && iseven(h))
            b = 1;
        end          
    case 86
        if ...
                (isequal(m,[1 1 0]) && iseven(h+k)) || ...                
                (isequal(m,[0 0 1]) && iseven(l)) || ...                
                (isequal(m,[1 0 0]) && iseven(h))
            b = 1;
        end          
    case 87
        if ...
                (isequal(m,[1 1 1]) && iseven(h+k+l)) || ...
                (isequal(m,[1 1 0]) && iseven(h+k)) || ...                
                (isequal(m,[0 1 1]) && iseven(k+l)) || ...
                (isequal(m,[1 1 1]) && iseven(l) && h==k) || ...
                (isequal(m,[0 0 1]) && iseven(l)) || ...                
                (isequal(m,[1 0 0]) && iseven(h))
            b = 1;
        end          
    case 88
        if ...
                (isequal(m,[1 1 1]) && iseven(h+k+l)) || ...
                (isequal(m,[1 1 0]) && iseven(h) && iseven(k)) || ...                
                (isequal(m,[0 1 1]) && iseven(k+l)) || ...
                (isequal(m,[1 1 1]) && iseven(l) && h==k) || ...
                (isequal(m,[0 0 1]) && iseven(l/2)) || ...                
                (isequal(m,[1 0 0]) && iseven(h)) || ...
                (isequal(m,[1 1 0]) && iseven(h) && h==-k)
            b = 1;
        end          
    case 89
        b = 1;
    case 90
        if ...
                (isequal(m,[1 0 0]) && iseven(h))
            b = 1;
        end        
    case 91
        if ...
                (isequal(m,[0 0 1]) && iseven(l/2))
            b = 1;
        end                  
    case 92
        if ...
                (isequal(m,[0 0 1]) && iseven(l/2)) || ...                
                (isequal(m,[1 0 0]) && iseven(h))
            b = 1;
        end                  
    case 93
        if ...
                (isequal(m,[0 0 1]) && iseven(l))
            b = 1;
        end                  
    case 94
        if ...
                (isequal(m,[0 0 1]) && iseven(l)) || ...                
                (isequal(m,[1 0 0]) && iseven(h))
            b = 1;
        end                  
    case 95
        if ...
                (isequal(m,[0 0 1]) && iseven(l/2))
            b = 1;
        end        
    case 96
        if ...
                (isequal(m,[0 0 1]) && iseven(l/2)) || ...                
                (isequal(m,[1 0 0]) && iseven(h))
            b = 1;
        end         
    case 97
        if ...
                (isequal(m,[1 1 1]) && iseven(h+k+l)) || ...
                (isequal(m,[1 1 0]) && iseven(h+k)) || ...                
                (isequal(m,[0 1 1]) && iseven(k+l)) || ...
                (isequal(m,[1 1 1]) && iseven(l) && h==k) || ...
                (isequal(m,[0 0 1]) && iseven(l)) || ...                
                (isequal(m,[1 0 0]) && iseven(h))
            b = 1;
        end         
    case 98
        if ...
                (isequal(m,[1 1 1]) && iseven(h+k+l)) || ...
                (isequal(m,[1 1 0]) && iseven(h+k)) || ...                
                (isequal(m,[0 1 1]) && iseven(k+l)) || ...
                (isequal(m,[1 1 1]) && iseven(l) && h==k) || ...
                (isequal(m,[0 0 1]) && iseven(l/2)) || ...                
                (isequal(m,[1 0 0]) && iseven(h))
            b = 1;
        end         
    case 99
        b = 1;
    case 100
        if ...
                (isequal(m,[0 1 1]) && iseven(k+l)) || ...
                (isequal(m,[1 0 0]) && iseven(h))
            b = 1;
        end         
    case 101
        if ...
                (isequal(m,[0 1 1]) && iseven(k+l)) || ...
                (isequal(m,[0 0 1]) && iseven(l))
            b = 1;
        end         
    case 102
        if ...
                (isequal(m,[0 1 1]) && iseven(k+l)) || ...
                (isequal(m,[0 0 1]) && iseven(l)) || ...
                (isequal(m,[1 0 0]) && iseven(h))
            b = 1;
        end         
    case 103
        if ...
                (isequal(m,[0 1 1]) && iseven(l)) || ...
                (isequal(m,[1 1 1]) && iseven(l) && h==k) || ...
                (isequal(m,[0 0 1]) && iseven(l))
              b = 1;
        end         
    case 104
        if ...
                (isequal(m,[0 1 1]) && iseven(k+l)) || ...
                (isequal(m,[1 1 1]) && iseven(l) && h==k) || ...
                (isequal(m,[0 0 1]) && iseven(l)) || ...                
                (isequal(m,[1 0 0]) && iseven(h))
            b = 1;
        end         
    case 105
        if ...
                (isequal(m,[1 1 1]) && iseven(l) && h==k) || ...
                (isequal(m,[0 0 1]) && iseven(l))
            b = 1;
        end         
    case 106
        if ...
                (isequal(m,[0 1 1]) && iseven(k)) || ...
                (isequal(m,[1 1 1]) && iseven(l) && h==k) || ...
                (isequal(m,[0 0 1]) && iseven(l)) || ...                
                (isequal(m,[1 0 0]) && iseven(h))
            b = 1;
        end         
    case 107
        if ...
                (isequal(m,[1 1 1]) && iseven(h+k+l)) || ...
                (isequal(m,[1 1 0]) && iseven(h+k)) || ...                
                (isequal(m,[0 1 1]) && iseven(k+l)) || ...
                (isequal(m,[1 1 1]) && iseven(l) && h==k) || ...
                (isequal(m,[0 0 1]) && iseven(l)) || ...
                (isequal(m,[1 0 0]) && iseven(h))
            b = 1;
        end         
    case 108
        if ...
                (isequal(m,[1 1 1]) && iseven(h+k+l)) || ...
                (isequal(m,[1 1 0]) && iseven(h+k)) || ...                
                (isequal(m,[0 1 1]) && iseven(k) && iseven(l)) || ...
                (isequal(m,[1 1 1]) && iseven(l) && h==k) || ...
                (isequal(m,[0 0 1]) && iseven(l)) || ...
                (isequal(m,[1 0 0]) && iseven(h))
            b = 1;
        end
    case 109
        if ...
                (isequal(m,[1 1 1]) && iseven(h+k+l)) || ...
                (isequal(m,[1 1 0]) && iseven(h+k)) || ...                
                (isequal(m,[0 1 1]) && iseven(k+l)) || ...
                (isequal(m,[1 1 1]) && iseven((2*h+l)/2) && h==k) || ...
                (isequal(m,[0 0 1]) && iseven(l/2)) || ...
                (isequal(m,[1 0 0]) && iseven(h)) || ...
                (isequal(m,[1 1 0]) && iseven(h) && h==-k)
            b = 1;
        end         
    case 110
        if ...
                (isequal(m,[1 1 1]) && iseven(h+k+l)) || ...
                (isequal(m,[1 1 0]) && iseven(h+k)) || ...                
                (isequal(m,[0 1 1]) && iseven(k) && iseven(l)) || ...
                (isequal(m,[1 1 1]) && iseven((2*h+l)/2) && h==k) || ...
                (isequal(m,[0 0 1]) && iseven(l/2)) || ...
                (isequal(m,[1 0 0]) && iseven(h)) || ...
                (isequal(m,[1 1 0]) && iseven(h) && h==-k)
            b = 1;
        end           
    case 111
        b = 1;
    case 112
        if ...
                (isequal(m,[1 1 1]) && iseven(l) && h==k) || ...
                (isequal(m,[0 0 1]) && iseven(l))
            b = 1;
        end   
    case 113
        if ...
                (isequal(m,[1 0 0]) && iseven(h))
            b = 1;
        end           
    case 114
        if ...
                (isequal(m,[1 1 1]) && iseven(l) && h==k) || ...
                (isequal(m,[0 0 1]) && iseven(l)) || ...
                (isequal(m,[1 0 0]) && iseven(h))
            b = 1;
        end           
    case 115
        b = 1;
    case 116
        if ...
                (isequal(m,[0 1 1]) && iseven(l)) || ...
                (isequal(m,[0 0 1]) && iseven(l))
            b = 1;
        end           
    case 117
        if ...
                (isequal(m,[0 1 1]) && iseven(k)) || ...
                (isequal(m,[1 0 0]) && iseven(h))
            b = 1;
        end           
    case 118
        if ...
                (isequal(m,[0 1 1]) && iseven(k+l)) || ...
                (isequal(m,[0 0 1]) && iseven(l)) || ...
                (isequal(m,[1 0 0]) && iseven(h))
            b = 1;
        end           
    case 119
        if ...
                (isequal(m,[1 1 1]) && iseven(h+k+l)) || ...
                (isequal(m,[1 1 0]) && iseven(h+k)) || ...                
                (isequal(m,[0 1 1]) && iseven(k+l)) || ...
                (isequal(m,[1 1 1]) && iseven(l) && h==k) || ...
                (isequal(m,[0 0 1]) && iseven(l)) || ...
                (isequal(m,[1 0 0]) && iseven(h))
            b = 1;
        end           
    case 120
        if ...
                (isequal(m,[1 1 1]) && iseven(h+k+l)) || ...
                (isequal(m,[1 1 0]) && iseven(h+k)) || ...                
                (isequal(m,[0 1 1]) && iseven(k) && iseven(l)) || ...
                (isequal(m,[1 1 1]) && iseven(l) && h==k) || ...
                (isequal(m,[0 0 1]) && iseven(l)) || ...
                (isequal(m,[1 0 0]) && iseven(h))
            b = 1;
        end           
    case 121
        if ...
                (isequal(m,[1 1 1]) && iseven(h+k+l)) || ...
                (isequal(m,[1 1 0]) && iseven(h+k)) || ...                
                (isequal(m,[0 1 1]) && iseven(k+l)) || ...
                (isequal(m,[1 1 1]) && iseven(l) && h==k) || ...
                (isequal(m,[0 0 1]) && iseven(l)) || ...
                (isequal(m,[1 0 0]) && iseven(h))
            b = 1;
        end           
    case 122
        if ...
                (isequal(m,[1 1 1]) && iseven(h+k+l)) || ...
                (isequal(m,[1 1 0]) && iseven(h+k)) || ...                
                (isequal(m,[0 1 1]) && iseven(k+l)) || ...
                (isequal(m,[1 1 1]) && iseven((2*h+l)/2) && h==k) || ...
                (isequal(m,[0 0 1]) && iseven(l/2)) || ...
                (isequal(m,[1 0 0]) && iseven(h)) || ...
                (isequal(m,[1 1 0]) && iseven(h) && h==-k)
            b = 1;
        end         
    case 123
        b = 1;
    case 124
        if ...
                (isequal(m,[0 1 1]) && iseven(l)) || ...
                (isequal(m,[1 1 1]) && iseven(l) && h==k) || ...
                (isequal(m,[0 0 1]) && iseven(l))
            b = 1;
        end          
    case 125
        if ...
                (isequal(m,[1 1 0]) && iseven(h+k)) || ...                
                (isequal(m,[0 1 1]) && iseven(k)) || ...
                (isequal(m,[1 0 0]) && iseven(h))
            b = 1;
        end          
    case 126
        if ...
                (isequal(m,[1 1 0]) && iseven(h+k)) || ...                
                (isequal(m,[0 1 1]) && iseven(k+l)) || ...
                (isequal(m,[1 1 1]) && iseven(l) && h==k) || ...
                (isequal(m,[0 0 1]) && iseven(l)) || ...
                (isequal(m,[1 0 0]) && iseven(h))
            b = 1;
        end          
    case 127
        if ...
                (isequal(m,[0 1 1]) && iseven(k)) || ...
                (isequal(m,[1 0 0]) && iseven(h))
            b = 1;
        end          
    case 128
        if ...
                (isequal(m,[0 1 1]) && iseven(k+l)) || ...
                (isequal(m,[1 1 1]) && iseven(l) && h==k) || ...
                (isequal(m,[0 0 1]) && iseven(l)) || ...
                (isequal(m,[1 0 0]) && iseven(h))
            b = 1;
        end          
    case 129
        if ...
                (isequal(m,[1 1 0]) && iseven(h+k)) || ...                
                (isequal(m,[1 0 0]) && iseven(h))
            b = 1;
        end          
    case 130
        if ...
                (isequal(m,[1 1 0]) && iseven(h+k)) || ...                
                (isequal(m,[0 1 1]) && iseven(l)) || ...
                (isequal(m,[1 1 1]) && iseven(l) && h==k) || ...
                (isequal(m,[0 0 1]) && iseven(l)) || ...
                (isequal(m,[1 0 0]) && iseven(h))
            b = 1;
        end          
    case 131
        if ...
                (isequal(m,[1 1 1]) && iseven(l) && h==k) || ...
                (isequal(m,[0 0 1]) && iseven(l))
            b = 1;
        end          
    case 132
        if ...
                (isequal(m,[0 1 1]) && iseven(l)) || ...
                (isequal(m,[0 0 1]) && iseven(l))
            b = 1;
        end          
    case 133
        if ...
                (isequal(m,[1 1 0]) && iseven(h+k)) || ...                
                (isequal(m,[0 1 1]) && iseven(k)) || ...
                (isequal(m,[1 1 1]) && iseven(l) && h==k) || ...
                (isequal(m,[0 0 1]) && iseven(l)) || ...
                (isequal(m,[1 0 0]) && iseven(h))
            b = 1;
        end          
    case 134
        if ...
                (isequal(m,[1 1 0]) && iseven(h+k)) || ...                
                (isequal(m,[0 1 1]) && iseven(k+l)) || ...
                (isequal(m,[0 0 1]) && iseven(l)) || ...
                (isequal(m,[1 0 0]) && iseven(h))
            b = 1;
        end          
    case 135
        if ...
                (isequal(m,[0 1 1]) && iseven(k)) || ...
                (isequal(m,[1 1 1]) && iseven(l) && h==k) || ...
                (isequal(m,[0 0 1]) && iseven(l)) || ...
                (isequal(m,[1 0 0]) && iseven(h))
            b = 1;
        end          
    case 136
        if ...
                (isequal(m,[0 1 1]) && iseven(k+l)) || ...
                (isequal(m,[0 0 1]) && iseven(l)) || ...
                (isequal(m,[1 0 0]) && iseven(h))
            b = 1;
        end          
    case 137
        if ...
                (isequal(m,[1 1 0]) && iseven(h+k)) || ...                
                (isequal(m,[1 1 1]) && iseven(l) && h==k) || ...
                (isequal(m,[0 0 1]) && iseven(l)) || ...
                (isequal(m,[1 0 0]) && iseven(h))
            b = 1;
        end          
    case 138
        if ...
                (isequal(m,[1 1 0]) && iseven(h+k)) || ...                
                (isequal(m,[0 1 1]) && iseven(l)) || ...
                (isequal(m,[0 0 1]) && iseven(l)) || ...
                (isequal(m,[1 0 0]) && iseven(h))
            b = 1;
        end          
    case 139
        if ...
                (isequal(m,[1 1 1]) && iseven(h+k+l)) || ...
                (isequal(m,[1 1 0]) && iseven(h+k)) || ...                
                (isequal(m,[0 1 1]) && iseven(k+l)) || ...
                (isequal(m,[1 1 1]) && iseven(l) && h==k) || ...
                (isequal(m,[0 0 1]) && iseven(l)) || ...
                (isequal(m,[1 0 0]) && iseven(h))
            b = 1;
        end          
    case 140
        if ...
                (isequal(m,[1 1 1]) && iseven(h+k+l)) || ...
                (isequal(m,[1 1 0]) && iseven(h+k)) || ...                
                (isequal(m,[0 1 1]) && iseven(k) && iseven(l)) || ...
                (isequal(m,[1 1 1]) && iseven(l) && h==k) || ...
                (isequal(m,[0 0 1]) && iseven(l)) || ...
                (isequal(m,[1 0 0]) && iseven(h))
            b = 1;
        end          
    case 141
        if ...
                (isequal(m,[1 1 1]) && iseven(h+k+l)) || ...
                (isequal(m,[1 1 0]) && iseven(h) && iseven(k)) || ...
                (isequal(m,[0 1 1]) && iseven(k+l)) || ...
                (isequal(m,[1 1 1]) && iseven((2*h+l)/2) && h==k) || ...
                (isequal(m,[0 0 1]) && iseven(l/2)) || ...
                (isequal(m,[1 0 0]) && iseven(h)) || ...
                (isequal(m,[1 1 0]) && iseven(h) && h==-k)
            b = 1;
        end          
    case 142
        if ...
                (isequal(m,[1 1 1]) && iseven(h+k+l)) || ...
                (isequal(m,[1 1 0]) && iseven(h) && iseven(k)) || ...
                (isequal(m,[0 1 1]) && iseven(k) && iseven(l)) || ...
                (isequal(m,[1 1 1]) && iseven((2*h+l)/2) && h==k) || ...
                (isequal(m,[0 0 1]) && iseven(l/2)) || ...
                (isequal(m,[1 0 0]) && iseven(h)) || ...
                (isequal(m,[1 1 0]) && iseven(h) && h==-k)
            b = 1;
        end        
    case 143
        b = 1;
    case 144
        if ...
                (isequal(m,[0 0 0 1]) && istriple(l))
            b = 1;
        end             
    case 145
        if ...
                (isequal(mi,[0 0 0 1]) && istriple(l))
            b = 1;
        end         
    case 146
        if ...
                (isequal(mi,[1 1 1 1]) && istriple(-h+k+l)) || ...
                (isequal(mi,[1 1 1 0]) && istriple(-h+k)) || ...
                (isequal(mi,[1 1 1 1]) && istriple(l) && h==k) || ...
                (isequal(mi,[1 1 0 1]) && istriple(h+l) && h==-k) || ...
                (isequal(mi,[0 0 0 1]) && istriple(l)) || ...
                (isequal(mi,[1 1 0 0]) && istriple(h) && h==-k)
            b = 1;
        end         
    case 147
        b = 1;
    case 148
        if ...
                (isequal(mi,[1 1 1 1]) && istriple(-h+k+l)) || ...
                (isequal(mi,[1 1 1 0]) && istriple(-h+k)) || ...
                (isequal(mi,[1 1 1 1]) && istriple(l) && h==k) || ...
                (isequal(mi,[1 1 0 1]) && istriple(h+l) && h==-k) || ...
                (isequal(mi,[0 0 0 1]) && istriple(l)) || ...
                (isequal(mi,[1 1 0 0]) && istriple(h) && h==-k)
            b = 1;
        end         
    case 149
        b = 1;
    case 150
        b = 1;
    case 151
        if ...
                (isequal(mi,[0 0 0 1]) && istriple(l))
            b = 1;
        end        
    case 152
        if ...
                (isequal(mi,[0 0 0 1]) && istriple(l))
            b = 1;
        end        
    case 153
        if ...
                (isequal(mi,[0 0 0 1]) && istriple(l))
            b = 1;
        end        
    case 154
        if ...
                (isequal(mi,[0 0 0 1]) && istriple(l))
            b = 1;
        end        
    case 155
        if ...
                (isequal(mi,[1 1 1 1]) && istriple(-h+k+l)) || ...
                (isequal(mi,[1 1 1 0]) && istriple(-h+k)) || ...
                (isequal(mi,[1 1 1 1]) && istriple(l) && h==k) || ...
                (isequal(mi,[1 1 0 1]) && istriple(h+l) && h==-k) || ...
                (isequal(mi,[0 0 0 1]) && istriple(l)) || ...
                (isequal(mi,[1 1 0 0]) && istriple(h) && h==-k)
            b = 1;
        end          
    case 156
        b = 1;
    case 157
        b = 1;
    case 158
        if ...
                (isequal(mi,[1 1 0 1]) && iseven(l) && h==-k) || ...
                (isequal(mi,[0 0 0 1]) && iseven(l))
            b = 1;
        end          
    case 159
        if ...
                (isequal(mi,[1 1 1 1]) && iseven(l) && h==k) || ...
                (isequal(mi,[0 0 0 1]) && iseven(l))
            b = 1;
        end          
    case 160
        if ...
                (isequal(mi,[1 1 1 1]) && istriple(-h+k+l)) || ...
                (isequal(mi,[1 1 1 0]) && istriple(-h+k)) || ...
                (isequal(mi,[1 1 1 1]) && istriple(l) && h==k) || ...
                (isequal(mi,[1 1 0 1]) && istriple(h+l) && h==-k) || ...
                (isequal(mi,[0 0 0 1]) && istriple(l)) || ...
                (isequal(mi,[1 1 0 0]) && istriple(h) && h==-k)
            b = 1;
        end         
    case 161
        if ...
                (isequal(mi,[1 1 1 1]) && istriple(-h+k+l)) || ...
                (isequal(mi,[1 1 1 0]) && istriple(-h+k)) || ...
                (isequal(mi,[1 1 1 1]) && istriple(l) && h==k) || ...
                (isequal(mi,[1 1 0 1]) && istriple(h+l) && iseven(l) && h==-k) || ...
                (isequal(mi,[0 0 0 1]) && istriple(l/2)) || ...
                (isequal(mi,[1 1 0 0]) && istriple(h) && h==-k)
            b = 1;
        end         
    case 162
        b = 1;
    case 163
        if ...
                (isequal(mi,[1 1 1 1]) && iseven(l) && h==k) || ...
                (isequal(mi,[0 0 0 1]) && iseven(l))
            b = 1;
        end         
    case 164
        b = 1;
    case 165
        if ...
                (isequal(mi,[1 1 0 1]) && iseven(l) && h==-k) || ...
                (isequal(mi,[0 0 0 1]) && iseven(l))
            b = 1;
        end        
    case 166
        if ...
                (isequal(mi,[1 1 1 1]) && istriple(-h+k+l)) || ...
                (isequal(mi,[1 1 1 0]) && istriple(-h+k)) || ...
                (isequal(mi,[1 1 1 1]) && istriple(l) && h==k) || ...
                (isequal(mi,[1 1 0 1]) && istriple(h+l) && h==-k) || ...
                (isequal(mi,[0 0 0 1]) && istriple(l)) || ...
                (isequal(mi,[1 1 0 0]) && istriple(h) && h==-k)
            b = 1;
        end         
    case 167
        if ...
                (isequal(mi,[1 1 1 1]) && istriple(-h+k+l)) || ...
                (isequal(mi,[1 1 1 0]) && istriple(-h+k)) || ...
                (isequal(mi,[1 1 1 1]) && istriple(l) && h==k) || ...
                (isequal(mi,[1 1 0 1]) && istriple(h+l) && iseven(l) && h==-k) || ...
                (isequal(mi,[0 0 0 1]) && istriple(l/2)) || ...
                (isequal(mi,[1 1 0 0]) && istriple(h) && h==-k)
            b = 1;
        end         
    case 168
        b = 1;
    case 169
        if ...
                (isequal(mi,[0 0 0 1]) && istriple(l/2))
            b = 1;
        end         
    case 170
        if ...
                (isequal(mi,[0 0 0 1]) && istriple(l/2))
            b = 1;
        end         
    case 171
        if ...
                (isequal(mi,[0 0 0 1]) && istriple(l))
            b = 1;
        end         
    case 172
        if ...
                (isequal(mi,[0 0 0 1]) && istriple(l))
            b = 1;
        end         
    case 173
        if ...
                (isequal(mi,[0 0 0 1]) && iseven(l))
            b = 1;
        end         
    case 174
        b = 1;
    case 175
        b = 1;
    case 176
        if ...
                (isequal(mi,[0 0 0 1]) && iseven(l))
            b = 1;
        end         
    case 177
        b = 1;        
    case 178
        if ...
                (isequal(mi,[0 0 0 1]) && istriple(l/2))
            b = 1;
        end        
    case 179
        if ...
                (isequal(mi,[0 0 0 1]) && istriple(l/2))
            b = 1;
        end        
    case 180
        if ...
                (isequal(mi,[0 0 0 1]) && istriple(l))
            b = 1;
        end        
    case 181
        if ...
                (isequal(mi,[0 0 0 1]) && istriple(l))
            b = 1;
        end         
    case 182
        if ...
                (isequal(mi,[0 0 0 1]) && iseven(l))
            b = 1;
        end         
    case 183
        b = 1;
    case 184
        if ...
                (isequal(mi,[1 1 1 1]) && iseven(l) && h==k) || ...
                (isequal(mi,[1 1 0 1]) && iseven(l) && h==-k) || ...
                (isequal(mi,[0 0 0 1]) && iseven(l))
            b = 1;
        end         
    case 185
        if ...
                (isequal(mi,[1 1 0 1]) && iseven(l) && h==-k) || ...
                (isequal(mi,[0 0 0 1]) && iseven(l))
            b = 1;
        end         
    case 186
        if ...
                (isequal(mi,[1 1 1 1]) && iseven(l) && h==k) || ...
                (isequal(mi,[0 0 0 1]) && iseven(l))
            b = 1;
        end         
    case 187
        b = 1;
    case 188
        if ...
                (isequal(mi,[1 1 0 1]) && iseven(l) && h==-k) || ...
                (isequal(mi,[0 0 0 1]) && iseven(l))
            b = 1;
        end         
    case 189
        b = 1;
    case 190
        if ...
                (isequal(mi,[1 1 1 1]) && iseven(l) && h==k) || ...
                (isequal(mi,[0 0 0 1]) && iseven(l))
            b = 1;
        end         
    case 191
        b = 1;
    case 192
        if ...
                (isequal(mi,[1 1 1 1]) && iseven(l) && h==k) || ...
                (isequal(mi,[1 1 0 1]) && iseven(l) && h==-k) || ...
                (isequal(mi,[0 0 0 1]) && iseven(l))
            b = 1;
        end          
    case 193
        if ...
                (isequal(mi,[1 1 0 1]) && iseven(l) && h==-k) || ...
                (isequal(mi,[0 0 0 1]) && iseven(l))
            b = 1;
        end          
    case 194
        if ...
                (isequal(mi,[1 1 1 1]) && iseven(l) && h==k) || ...
                (isequal(mi,[0 0 0 1]) && iseven(l))
            b = 1;
        end          
    case 195
        b = 1;
    case 196
        if ...
                (isequal(m,[1 1 1]) && iseven(h+k) && iseven(h+l)  && iseven(k+l)) || ...
                (nnz(miller)==2 && iseven(h) && iseven(k) && iseven(l)) || ...
                (nnz(miller)==3 && numel(unique(miller))==2 && iseven(sum(unique(miller)))) || ...
                (nnz(miller)==1 && iseven(sum(miller)))
            b = 1;
        end
    case 197
        if ...
                iseven(h+k+l)
            b = 1;
        end        
    case 198
        if ...
                nnz(miller)==1 && iseven(sum(miller))
            b = 1;
        end
    case 199
        if ...
                iseven(h+k+l)
            b = 1;
        end         
    case 200
        b = 1;
    case 201
        if ...
                nnz(miller)~=3 && iseven(sum(miller))
            b = 1;
        end
    case 202
        if ...
                iseven(h+k) && iseven(h+l) && iseven(k+l)
            b = 1;
        end
    case 203
        if ...
                (isequal(m,[1 1 1]) && iseven(h+k) && iseven(h+l)  && iseven(k+l)) || ...
                (nnz(miller)==2 && iseven(sum(miller)/2) && iseven(h) && iseven(k) && iseven(l)) || ...
                (nnz(miller)==3 && numel(unique(miller))==2 && iseven(sum(unique(miller)))) || ...
                (nnz(miller)==1 && iseven(sum(miller)/2))
            b = 1;
        end
    case 204
        if ...
                iseven(h+k+l)
            b = 1;
        end
    case 205
        if ...
                (isequal(m,[0 1 1]) && iseven(k)) || ...
                (isequal(m,[1 0 1]) && iseven(l)) || ...
                (isequal(m,[1 1 0]) && iseven(h)) || ...
                (nnz(miller)==1 && iseven(sum(miller)))
            b = 1;
        end        
    case 206
        if ...
                (isequal(m,[1 1 1]) && iseven(sum(miller))) || ...
                (nnz(miller)==2 && iseven(h) && iseven(k) && iseven(l)) || ...
                (nnz(miller)==3 && iseven(l) && h==k ) || ...
                (nnz(miller)==3 && iseven(h) && k==l ) || ...
                (nnz(miller)==3 && iseven(k) && l==h ) || ...
                (nnz(miller)==1 && iseven(sum(miller)))
            b = 1;
        end        
    case 207
        b = 1;
    case 208
        if ...
                nnz(miller)==1 && iseven(sum(miller)) 
            b = 1;
        end
    case 209
        if ...
                iseven(h+k) && iseven(h+l) && iseven(k+l)
            b = 1;
        end        
    case 210
        if ...
                (isequal(m,[1 1 1]) && iseven(h+k) && iseven(h+l)  && iseven(k+l)) || ...
                (nnz(miller)==2 && iseven(h) && iseven(k) && iseven(l)) || ...
                (nnz(miller)==3 && numel(unique(miller))==2 && iseven(sum(unique(miller)))) || ...
                (nnz(miller)==1 && iseven(sum(miller)/2))
            b = 1;
        end        
    case 211
        if ...
                iseven(h+k+l)
            b = 1;
        end        
    case 212
        if ...
                nnz(miller)==1 && iseven(sum(miller)/2) 
            b = 1;
        end        
    case 213
        if ...
                nnz(miller)==1 && iseven(sum(miller)/2) 
            b = 1;
        end         
    case 214
        if ...
                (isequal(m,[1 1 1]) && iseven(h+k+l)) || ...
                (nnz(miller)==2 && iseven(sum(miller))) || ...
                (nnz(miller)==3 && iseven(l) && h==k ) || ...
                (nnz(miller)==3 && iseven(h) && k==l ) || ...
                (nnz(miller)==3 && iseven(k) && l==h ) || ...
                (nnz(miller)==1 && iseven(sum(miller)/2))
            b = 1;
        end         
    case 215
        b = 1;
    case 216
        if ...
                iseven(h+k) && iseven(h+l) && iseven(k+l)
            b = 1;
        end         
    case 217
        if ...
                iseven(h+k+l)
            b = 1;
        end         
    case 218
        if ...
                (nnz(miller)==3 && iseven(l) && h==k ) || ...
                (nnz(miller)==3 && iseven(h) && k==l ) || ...
                (nnz(miller)==3 && iseven(k) && l==h ) || ...
                (nnz(miller)==1 && iseven(sum(miller)))
            b = 1;
        end         
    case 219
        if ...
                (isequal(m,[1 1 1]) && iseven(h+k) && iseven(k+l) && iseven(h+l)) || ...
                (nnz(miller)==2 && iseven(h)  && iseven(k) && iseven(l)) || ...
                (nnz(miller)==3 && numel(unique(miller))==2 && iseven(h)  && iseven(k) && iseven(l)) || ...
                (nnz(miller)==1 && iseven(sum(miller)))
            b = 1;
        end         
    case 220
        if ...
                (isequal(m,[1 1 1]) && iseven(h+k+l)) || ...
                (nnz(miller)==2 && iseven(sum(miller))) || ...
                (nnz(miller)==3 && numel(unique(miller))==2 && iseven(sum(miller)/2)) || ...
                (nnz(miller)==1 && iseven(sum(miller)))
            b = 1;
        end        
    case 221
        b = 1;
    case 222
        if ...
                (nnz(miller)==2 && iseven(sum(miller))) || ...
                (nnz(miller)==3 && iseven(l) && h==k ) || ...
                (nnz(miller)==3 && iseven(h) && k==l ) || ...
                (nnz(miller)==3 && iseven(k) && l==h ) || ...
                (nnz(miller)==1 && iseven(sum(miller)))
            b = 1;
        end         
    case 223
        if ...
                (nnz(miller)==3 && iseven(l) && h==k ) || ...
                (nnz(miller)==3 && iseven(h) && k==l ) || ...
                (nnz(miller)==3 && iseven(k) && l==h ) || ...
                (nnz(miller)==1 && iseven(sum(miller)))
            b = 1;
        end         
    case 224
        if ...
                (nnz(miller)==2 && iseven(sum(miller))) || ...
                (nnz(miller)==1 && iseven(sum(miller)))
            b = 1;
        end         
    case 225
        if ...
                iseven(h+k) && iseven(h+l) && iseven(k+l)
            b = 1;
        end         
    case 226
        if ...
                iseven(h+k) && iseven(h+l) && iseven(k+l)
            b = 1;
        end         
    case 227
        if ...
                (isequal(m,[1 1 1]) && iseven(h+k) && iseven(h+l)  && iseven(k+l)) || ...
                (nnz(miller)==2 && iseven(sum(miller)/2) && iseven(h) && iseven(k) && iseven(l)) || ...
                (nnz(miller)==3 && numel(unique(miller))==2 && iseven(sum(unique(miller)))) || ...
                (nnz(miller)==1 && iseven(sum(miller)/2))
            b = 1;
        end          
    case 228
        if ...
                (isequal(m,[1 1 1]) && iseven(h+k) && iseven(h+l)  && iseven(k+l)) || ...
                (nnz(miller)==2 && iseven(sum(miller)/2) && iseven(h) && iseven(k) && iseven(l)) || ...
                (nnz(miller)==3 && numel(unique(miller))==2 && iseven(h) && iseven(k) && iseven(l)) || ...
                (nnz(miller)==1 && iseven(sum(miller)/2))
            b = 1;
        end          
    case 229
        if ...
                iseven(h+k+l)
            b = 1;
        end          
    case 230
        if ...
                (isequal(m,[1 1 1]) && iseven(h+k+l)) || ...
                (nnz(miller)==2 && iseven(h) && iseven(k) && iseven(l)) || ...
                (nnz(miller)==3 && numel(unique(miller))==2 && iseven(sum(miller)/2)) || ...
                (nnz(miller)==1 && iseven(sum(miller)/2))
            b = 1;
        end         
end


function y=iseven(n)
y = ~mod(n,2);


function y=istriple(n)
y = ~mod(n,3);